<?php
/**
* @version		1.1.0
* @package		Ambitionality
* @copyright	Copyright (C) 2008 Ambitionality Software LLC. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* Joomla! is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Seach utility for the Simplest Forum.
*/
class plgSearchSimplestForum extends JPlugin
{
    /**
     * Called by the search component when a search is performed.
     * @param $text the search text
     * @param $phrase the way to search the text (exact, all, any)
     * @param $ordering the ordering of the results
     * @param $areas doesn't matter to us
    */
    function onSearch($text, $phrase = '', $ordering = '', $areas = null)
    {
        // get the db and default params
        $db = &JFactory::getDBO();
        $params = &JComponentHelper::getParams('com_simplestforum');
        $defaultViewGID = $params->get('viewgid');

        require_once(JPATH_SITE.DS.'components'.DS.'com_simplestforum'.DS.'helpers'.DS.'forum.php');

        // if we don't have anything to search for our plugin, let's bail out
        if (is_array($areas)) {
            if (!array_intersect($areas, array_keys( $this->onSearchAreas()))) {
                return array();
            }
        }

        // if there is not text to search, bail out rather than returning everything
        $text = trim($text);
        if ($text == '') {
            return array();
        }

        // build the where clause based on the search phrase
        switch ($phrase) {
            case 'exact':
                $where = '(a.message LIKE '.$db->Quote('%'.$text.'%').' OR a.subject LIKE '.$db->Quote('%'.$text.'%').')';
                break;
            case 'all':
            case 'any':
            default:
                $words = explode(' ', $text);
                $wheres2 = array();
                foreach ($words as $word) {
                    $wheres2[] = '(a.message REGEXP '.$db->Quote('[[:<:]]'.$word.'[[:>:]]').' OR a.subject REGEXP '.$db->Quote('[[:<:]]'.$word.'[[:>:]]').')';
                }
                $where = '('.implode(($phrase == 'all'?' AND ':' OR '), $wheres2).')';
                break;
        }

        switch ($ordering) {
            case 'oldest':
                $order = 'a.date ASC';
                break;
            case 'alpha':
                $order = 'a.subject ASC';
                break;
            case 'category':
            case 'popular':
            case 'newest':
            default:
                $order = 'a.date DESC';
        }

        $user = &JFactory::getUser();

        // build and execute the query
        $query = 'SELECT a.subject AS title, a.message AS text, a.date, a.forumId, 0 AS browsernav, NULL as section, a.date AS created, a.thread
            FROM #__simplestforum_post AS a
            LEFT JOIN #__simplestforum_forum AS b ON b.id = a.forumId
            WHERE '.$where.' AND IF(b.viewgid IS NOT NULL, b.viewgid, '.$db->Quote($defaultViewGID).') <= '.$db->Quote($user->get('gid')).'
            ORDER BY '.$order
        ;
        $db->setQuery($query);
        $rows = ($rows = $db->loadObjectList())?$rows:array();

        // build the links
        foreach ($rows as $row) {
            $row->href = JRoute::_('index.php?option=com_simplestforum&view=postlist&forumId='.$row->forumId.'&parentId='.$row->thread);
        }

        return $rows;
    } //end onSearch

    /** 
     * Called to populate the search component page's options.
     * @return an array of areas that will be matched by this search utility
    */
    function onSearchAreas()
    {
        static $areas = array(
            'simplestforumposts' => 'Forum Posts'
        );

        return $areas;
    } //end onSearchAreas

} //end class
?>
