<?php
/**
* @version		1.0.0
* @package		Ambitionality
* @copyright	Copyright (C) 2008 Ambitionality Software LLC. All rights reserved.
* @license		GNU/GPL 2.0, see LICENSE.php
* Joomla! is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
* This component uses the simplestforum_extension_attributes table as follows
* to store the subscriptions:
*     extension: simplestforumnotify
*     aux1: threadId
*     aux2: userId
*     aux4: unsubscribe hash
*/
class plgComponentSimplestForumNotify extends JPlugin
{
    /**
     * This is a constructor to hack the params in just in case they aren't
     * automatically loaded.
    */
    function plgComponentSimplestForumNotify(&$subject, $config = array())
    {
        parent::__construct($subject);

        // fix  / hack for certain versions of PHP / MySQL that die on trying
        // to load the params automatically through the Joomla! base
        if (!$this->params) {
             $db = &JFactory::getDBO();
             $query = 'SELECT a.params
                       FROM `#__plugins` AS a
                       WHERE a.element = '.$db->Quote('simplestforumnotify').' AND a.folder = '.$db->Quote('component')
             ;
             $db->setQuery($query);
             $params = $db->loadResult();
             $this->params = new JParameter($params);
         }

         // load the language file
         $lang = &JFactory::getLanguage();
         $lang->load('plg_component_simplestforumnotify', JPATH_SITE.DS.'administrator');
    } //end constructor

    /**
     * Called directly before the post renders so that any modifications to
     * formatting, etc. can be done.
     * @param $post the post object
    */
    function onBeforeRenderPost(&$post)
    {
        $user = &JFactory::getUser();
        if ($this->params->get('allowsubscribers')) {
            if (!isset($post->buttons)) {
                $post->buttons = array();
            }

            if (!$user->get('guest')) {
                $post->buttons[] = '<a href="'.JRoute::_('index.php?option=com_simplestforum&task=extension&extension=simplestforumnotify&extensionTask=subscribe&thread='.$post->thread).'">'.JText::_('SUBSCRIBE TO THIS THREAD').'</a>';
            } else {
                $post->buttons[] = '<span title="'.JText::_('LOG IN REQUIRED TO SUBSCRIBE').'">'.JText::_('SUBSCRIBE TO THIS THREAD').'</span>';
            }
        }
    } //end onBeforeRenderPost


    /**
     * This is called just after a post is successfully saved. We use this to
     * email out notifications to any subscribers of this post's thread.
     * @param $post the post table object
     * @param $postFiles the post _FILES array as received when the post was submitted
     * @param $rawText the RAW text from the message post (before any safety transformations are performed)
     * @param $error an array to which any errors should be pushed
    */
    function onAfterSavePost(&$post, &$postFiles, &$rawText, &$error)
    {
        // send email notifications to administrators
        if (($this->params->get('notifyontopic') && $newTopic) || $this->params->get('notifyonpost')) {
            $config = &JFactory::getConfig();
            $siteAdminEmail = $config->getValue('mailfrom');

            $this->_emailNotification($post, $siteAdminEmail);
        }


        // now we will email subscribers
        $db = &JFactory::getDBO();
        $query = 'SELECT a.*, b.email
                  FROM #__simplestforum_extension_attributes AS a
                  LEFT JOIN #__users AS b ON b.id = a.aux2
                  WHERE a.extension = '.$db->Quote('simplestforumnotify').' AND a.aux1 = '.(int)$post->thread
        ;
        $db->setQuery($query);
        $items = ($items = $db->loadObjectList())?$items:array();

        // if there are not subscribers, bail out
        if (empty($items)) {
            return;
        }

        foreach ($items as $item) {
            // ensure that they actually have an email address
            if (isset($item->email) && strlen($item->email)) {
                $this->_emailNotification($post, $item->email, $item->id, $item->aux4);
            }
        }
    } //end onAfterSavePost

    /**
     * Emails out a notification message for the specified post and user information. 
     * @param $post the post that was submitted that caused this notification
     * @param $userEmail the string email address of the recipient / subscriber
     * @param $unsubscribeId the id of the subscription that this message is
     * being sent in connection with
     * @param $unsubscribeHash a unique hash that helps prevent unauthorized
     * users from unsubscribing someone
    */
    function _emailNotification($post, $userEmail, $unsubscribeId = null, $unsubscribeHash = null)
    {
        $config = &JFactory::getConfig();
        $siteAdminEmail = $config->getValue('mailfrom');
        $siteName = $config->getValue('fromname');

        $subject = $this->params->get('notificationsubject');
        $message = $this->params->get('notificationmessage')."\n\n".$post->get('subject')."\n\n".$post->get('message')."\n\n".JURI::base().JRoute::_('index.php?option=com_simplestforum&view=postlist&parentId='.$post->id.'&forumId='.$post->forumId, false);
        
        // if there is a subscription associated with this, attach the unsubscribe message
        if ($unsubscribeId) {
            $message .= "\n\n".$this->params->get('unsubscribemessage')."\n".JURI::base().JRoute::_('index.php?option=com_simplestforum&task=extension&extension=simplestforumnotify&extensionTask=unsubscribe&id='.$unsubscribeId.'&hash='.$unsubscribeHash, false);
        }

        JUtility::sendMail($siteAdminEmail, $siteName, $userEmail, $subject, $message, 0);
    } //end _emailNotification

    /**
     * Called when someone attempts to subscribe or unsubscribe. This handles
     * the request much like a custom controller would.
     * @param $extension the string name of the extension that initiated the task
     * @param $task the string name of the task that was initiated
     * @param $postData an array of the data included with the request
    */
    function onExtensionTask($extension, $task, &$postData)
    {
        // only handle events from this extension
        if ($extension != 'simplestforumnotify') {
            return;
        }

        switch ($task) {
            case 'subscribe':
                $this->_subscribe($postData);
                break;
            case 'unsubscribe':
                $this->_unsubscribe($postData);
                break;
        }
    } //end onExtensionTask

    /**
     * Performs a new subscribe based on the posted information.
     * @param $postData the data that was posted in the request.
    */
    function _subscribe(&$postData)
    {
        // if subscribers are not allowed, don't do anything
        if (!$this->params->get('allowsubscribers')) {
            $mainframe->enqueueMessage(JText::_('ERROR SUBSCRIPTIONS NOT ALLOWED'), 'error');
            return;
        }

        // if a guest attempts to do this somehow, let them know it's not allowed
        $mainframe = &JFactory::getApplication();
        $user = &JFactory::getUser();
        if ($user->get('guest')) {
            $mainframe->enqueueMessage(JText::_('ERROR ONLY REGISTERED MAY SUBSCRIBE'), 'error');
            return;
        }

        // get the thread to subscribe
        $thread = JFilterInput::clean($postData['thread'], 'int');

        // check to see if the user is already subscribed to this thread, if
        // so, quietly quit what we are doing
        $db = &JFactory::getDBO();
        $query = 'SELECT COUNT(*)
                  FROM #__simplestforum_extension_attributes AS a
                  WHERE a.aux1 = '.(int)$thread.' AND a.aux2 = '.(int)$user->get('id')
        ;
        $db->setQuery($query);
        if ($db->loadResult()) {
            return;
        }

        // save the subscription
        JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_simplestforum'.DS.'tables');
        $table = &JTable::getInstance('extensionattributes', 'SimplestForumTable');
        $table->set('extension', 'simplestforumnotify');
        $table->set('aux1', $thread);
        $table->set('aux2', $user->get('id'));
        $table->set('aux4', JUtility::getHash($user->get('email').time()));

        if (!$table->store()) {
            $mainframe->enqueueMessage(JText::_('ERROR SUBSCRIBING'), 'error');
            return;
        }

        $mainframe->enqueueMessage(JText::_('SUBSCRIBED SUCCESSFULLY'));
    } //end _subscribe

    /**
     * Performs the unsubscribe action.
     * @param $postData the data posted in the request.
    */
    function _unsubscribe(&$postData)
    {
        $mainframe = &JFactory::getApplication();

        // get the subscription id and unique hash associated with it
        $id = JFilterInput::clean($postData['id'], 'int');
        $hash = JFilterInput::clean($postData['hash'], 'string');

        // attempt to find the corresponding subscription in the database
        JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_simplestforum'.DS.'tables');
        $table = &JTable::getInstance('extensionattributes', 'SimplestForumTable');
        if (!$table->load((int)$id)) {
            $mainframe->enqueueMessage(JText::_('ERROR UNSUBSCRIBING'));
            return;
        }

        // if we were given a valid hash and id, go ahead and delete the subscription
        if ($hash == $table->get('aux4') && $table->delete()) {
            $mainframe->enqueueMessage(JText::_('SUBSCRIPTION REMOVED'));
            return;
        }

        $mainframe->enqueueMessage(JText::_('ERROR UNSUBSCRIBING'));
    } //end _unsubscribe

} //end class
?>
